import os
import datetime
import hmac
import hashlib
import urllib3
urllib3.disable_warnings()
import requests
import pandas as pd
import csv
from dateutil import parser


import xml.etree.ElementTree as et

from measurements.sources.base import BaseSource
from measurements.utils import strong_float

class DavisAPI(BaseSource):
    def __init__(self, tz='Europe/Rome'):
        self.baseurl = "http://www.meteosystem.com/wlip/"
        self.tz = tz

    def get_df(self, code, last=1):
        url = self.baseurl + code + '.php'
        r = requests.get(url)
        decoded_content = r.content.decode('utf-8')
        reader = csv.reader(decoded_content.splitlines(),
                            delimiter='|',
                            doublequote=False,
                            quoting=csv.QUOTE_NONE)
        data = []
        for j, row in enumerate(reader):
            print(j, row)
            data_str = row[1] + ' ' + row[0].replace('ERR', '').replace('.', ':')
            _datetime = parser.parse(data_str, dayfirst=True)
            print(_datetime)
            r = {'datetime': _datetime,
                 'TempAria': strong_float(row[2]),
                 'UmidAriaRel': strong_float(row[3]),
                 'PntRugiada': strong_float(row[4]),
                 'IndCalore': strong_float(row[5]),
                 'IndRaffr': strong_float(row[6]),
                 'PressAtm': strong_float(row[7]),
                 'RadSol': strong_float(row[8]),
                 'TempAriaMin': strong_float(row[11]),
                 'TempAriaMax': strong_float(row[14]),
                 'VelVento': strong_float(row[44]),
                 'VelVentoMed10min': strong_float(row[46]),
                 'RafficaVento': strong_float(row[47]),
                 'Precip': strong_float(row[49]),
                 'PercPrecipMax': strong_float(row[50]),
                 'PrecipGiorno': strong_float(row[54]),
                 'PrecipMese': strong_float(row[56]),
                 'PrecipAnno': strong_float(row[58])
                 }
            data.append(r)

        # return True
        self.df = pd.DataFrame(data)
        self.df.set_index('datetime', inplace=True)
        self.df.index = pd.to_datetime(self.df.index)

        # localize datetime index
        self.df.index = self.df.index.tz_localize(self.tz)
        return self.df





from django.urls import path, include
from .writeapi import  write
from .views import locations_flatjson, measure_geojson

urlpatterns = [
    path('write', write),
    path('locations/flatjson', locations_flatjson),
    path('measures/geojson', measure_geojson),
]


# Generated by Django 2.2.1 on 2019-05-28 20:09

import django.contrib.gis.db.models.fields
import django.contrib.postgres.fields
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager
import psqlextra.manager.manager


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Location',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(max_length=150)),
                ('geo', django.contrib.gis.db.models.fields.PointField(null=True, srid=4326)),
            ],
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('extra', psqlextra.manager.manager.PostgresManager()),
            ],
        ),
        migrations.CreateModel(
            name='Network',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=100)),
                ('label', models.CharField(blank=True, max_length=150, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Parameter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(blank=True, max_length=100, null=True)),
                ('uri', models.URLField(blank=True, null=True)),
                ('label', models.CharField(max_length=150)),
            ],
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('extra', psqlextra.manager.manager.PostgresManager()),
            ],
        ),
        migrations.CreateModel(
            name='Sensor',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=100)),
                ('label', models.CharField(blank=True, max_length=150, null=True)),
            ],
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('extra', psqlextra.manager.manager.PostgresManager()),
            ],
        ),
        migrations.CreateModel(
            name='SourceType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=100, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='Station',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=100)),
                ('label', models.CharField(blank=True, max_length=150, null=True)),
                ('location', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='measurements.Location')),
                ('network', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='measurements.Network')),
                ('source', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='measurements.SourceType')),
            ],
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('extra', psqlextra.manager.manager.PostgresManager()),
            ],
        ),
        migrations.CreateModel(
            name='Serie',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('stats_mean', models.FloatField(null=True)),
                ('stats_outliers', django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), null=True, size=None)),
                ('parameter', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='measurements.Parameter')),
                ('sensor', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='measurements.Sensor')),
                ('station', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='measurements.Station')),
            ],
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('extra', psqlextra.manager.manager.PostgresManager()),
            ],
        ),
        migrations.CreateModel(
            name='Measure',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('timestamp', models.DateTimeField(db_index=True)),
                ('value', models.FloatField()),
                ('serie', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='measurements.Serie')),
            ],
            managers=[
                ('objects', django.db.models.manager.Manager()),
                ('extra', psqlextra.manager.manager.PostgresManager()),
            ],
        ),
    ]



from django.core.management.base import BaseCommand

# from meteo.pgutils import load_data
from measurements.settings import SOURCE_AUTH
from measurements.sources.pessl import PesslAPI
from measurements.sources.mtt import MttAPI
from measurements.models import SourceType, Measure

a = MttAPI()
a.get_df(code='T0193')


from psqlextra.query import ConflictAction


ps = SourceType.objects.get(code='pessl')
for s in ps.station_set.all():
    keys = SOURCE_AUTH['pessl'][s.network.code]
    print(keys)
    pesslapi = PesslAPI(keys['public_key'],
                            keys['private_key'])
    df = pesslapi.get_df(s.code, 1000)
    print(df)



a = df[['HC Air temperature|avg']].copy()
a.reset_index(inplace=True)
a.columns = ['timestamp', 'value']
a['serie_id'] = 2

Measure.extra.on_conflict(['serie_id', 'timestamp', 'value'],
                          ConflictAction.UPDATE).bulk_insert(
    a.to_dict(orient='record'))

from measurements.sources.davis import DavisAPI

ds = DavisAPI()
a = ds.get_df('cantineferrari/ferrari1')


from django.conf import settings
from measurements.sources.elmed import ElmedAPI
elmed = ElmedAPI(None, settings.MEASUREMENTS_SOURCE_AUTH['elmed']['ferrari']['private_key'])
df = elmed.get_df(88)


from django.core.management.base import BaseCommand

# from meteo.pgutils import load_data
from measurements.settings import SOURCE_AUTH
from measurements.sources.davis import DavisAPI
from measurements.models import SourceType
from measurements.utils import get_serie, load_serie

PARAMETER_MAP = {"TempAria": "At_temp",
                 # "nasstemp": "At_temp_WetBulb",
                 # "UmidAriaRel": "RelHumidity",
                 "PntRugiada": "DewPoint",
                 "PressAtm": "AtPres",
                 "RadSol": "GlobalRad",
                 "VelVento": "WindSpeed",
                 "Precip": "Precipitation"
                 }

class Command(BaseCommand):
    help = "Command to import Davis data"

    def handle(self, *args, **options):
        ps = SourceType.objects.get(code='davis')
        for s in ps.station_set.filter(status='active'):
            davisapi = DavisAPI()
            df = davisapi.get_df(s.code, 5)

            for k, v in PARAMETER_MAP.items():
                if k in df.columns:
                    serie = get_serie(s, v)
                    load_serie(df[k].copy(), serie.id)


PARAMETERS_BASE = [
    ("Precip", "Precipication"),
    ("AtTemp", "Atmospheric temperature"),
    ("AtPres", "Atmospheric pressure"),
    ("RelHumidity", "Relative humidity"),
    ("LeafWet", "Leaf wetness"),
    ("DewPoint", "Dew point"),
    ("WindSpeed", "Wind speed"),
    ("SoilMoisture", "Soil moisture"),
    ("SoilTemp", "Soil temperature"),
    ("GlobalRad", "Global radiation"),
]



import os
import io
import datetime
import hmac
import hashlib
import urllib3
urllib3.disable_warnings()
import requests
import pandas as pd
from Crypto.Hash import HMAC
from Crypto.Hash import SHA256


from measurements.sources.base import BaseSource

class ElmedAPI(BaseSource):
    def __init__(self, public_key, private_key, tz='Europe/Rome'):
        self.baseurl = "http://www.elmedweb.com/MeteoWeb/download/download.php?"
        self.public_key = public_key
        self.private_key = private_key
        self.tz = tz


    def get_df(self, code, last=1):
        end_date = datetime.datetime.today()
        start_date = end_date - datetime.timedelta(days=last)
        payload = {"stationnr[]": code,
                   "auth": self.private_key,
                   "format": "csv_large",
                   "start_date": start_date.strftime('%d-%m-%Y'),
                   "end_date": end_date.strftime('%d-%m-%Y')
                   }
        r = requests.get(self.baseurl, params=payload)
        df = pd.read_csv(io.StringIO(r.text))

        # set datetime index
        df.Datum = pd.to_datetime(df.Datum, format="%Y%m%d %H:%M:%S")
        df.set_index('Datum', inplace=True)
        self.df = df

        self.df.index = self.df.index.tz_localize(self.tz)
        return df



# Generated by Django 2.2.1 on 2019-06-21 15:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('measurements', '0005_serie_height'),
    ]

    operations = [
        migrations.AddField(
            model_name='station',
            name='status',
            field=models.CharField(choices=[('active', 'active'), ('decomissioned', 'decomissioned'), ('in maintenance', 'in maintenance')], default='active', max_length=255),
        ),
    ]



from measurements.settings import DATABASE_ROUNTING

class MeasurementsRouter:
    """
    A router to control all database operations on models in the
    measurements application.
    """
    def db_for_read(self, model, **hints):
        """
        Attempts to read measurements models go to measurements_db.
        """
        if model._meta.app_label == 'measurements':
            return DATABASE_ROUNTING
        return None

    def db_for_write(self, model, **hints):
        """
        Attempts to write measurements models go to measurements_db.
        """
        if model._meta.app_label == 'measurements':
            return DATABASE_ROUNTING
        return None

    def allow_relation(self, obj1, obj2, **hints):
        """
        Allow relations if a model in the measurements app is involved.
        """
        if obj1._meta.app_label == 'measurements' or \
           obj2._meta.app_label == 'measurements':
           return True
        return None

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        """
        Make sure the measurements app only appears in the 'measurements_db'
        database.
        """
        if app_label == 'measurements':
            return db == DATABASE_ROUNTING
        return None


from django.core.management.base import BaseCommand

# from meteo.pgutils import load_data
from measurements.settings import SOURCE_AUTH
from measurements.sources.mtt import MttAPI
from measurements.models import SourceType
from measurements.utils import get_serie, load_serie


PARAMETER_MAP = {"temperatura": "AtTemp",
                 "pioggia": "Precipitation",
                 # "d": "WindDirFrom",
                 "v": "WindSpeed",
                 "rh": "RelHumidity",
                 "rsg": "GlobalRad"
                 }


class Command(BaseCommand):
    help = "Command to import Meteotrentino data"

    def handle(self, *args, **options):
        ps = SourceType.objects.get(code='mtt')
        for s in ps.station_set.filter(status='active'):
            apiclient = MttAPI()
            df = apiclient.get_df(s.code)

            for k, v in PARAMETER_MAP.items():
                if k in df.columns:
                    serie = get_serie(s, v)
                    load_serie(df[k].copy(), serie.id)



import os
import datetime
import hmac
import hashlib
import urllib3
urllib3.disable_warnings()
import requests
import pandas as pd
from Crypto.Hash import HMAC
from Crypto.Hash import SHA256


from measurements.sources.base import BaseSource

class PesslAPI(BaseSource):
    def __init__(self, public_key, private_key, tz='Etc/GMT-1'):
        self.baseurl = "https://api.fieldclimate.com"
        self.public_key = public_key
        self.private_key = private_key
        self.tz = tz

    def get_auth_headers(self, request, method="GET"):
        d = datetime.datetime.utcnow()
        timestamp = d.strftime('%a, %d %b %Y %H:%M:%S GMT')

        # Creating content to sign with private key
        msg = "{}{}{}{}".format(method, request,
                                        timestamp, self.public_key).encode(encoding='utf-8')

        # msg = (self._method + self._apiRoute + dateStamp + self._publicKey).encode(encoding='utf-8')
        h = HMAC.new(self.private_key.encode(encoding='utf-8'), msg, SHA256)
        signature = h.hexdigest()

        # Hash content to sign into HMAC signature
        # signing = hmac.new(self.private_key,
        #                   content_to_sign,
        #                   hashlib.sha256)

        headers = {
            "Accept": "application/json",
            "Authorization": "hmac {}:{}".format(self.public_key,
                                                 signature),
            "Date": "{}".format(timestamp)
        }
        return headers

    def _todf(self, data):
        _colindex = []
        columns = []
        _data = {}
        for sk, s in data['data'].items():
            for aggk, agg in s['aggr'].items():
                _key = "{}_{}".format(sk, aggk)
                columns.append(_key)
                _data[_key] = agg
                _colindex.append((s['name'], aggk))
        colindex = pd.MultiIndex.from_tuples(_colindex, names=['datetime', 'agg'])
        # gmt1 to utc
        # print data['dates']
        # ATTENZIONE: provvisoriamente bisogna forzare un shift di 1 ora per un problema alle API di pessl
        # index = pd.to_datetime(data['dates'])
        # Ora il bug e' stato sistemato lato Pessl
        index = pd.to_datetime(data['dates']) # + pd.to_timedelta(1, 'h')
        # print index
        # index = pd.to_datetime(index, unit='ms')
        df = pd.DataFrame(_data, columns=columns, index=index, dtype='float')
        df.columns = colindex
        return df


    def _get_data(self, code, route='pretty', last=1):
        """
        route: pretty, data, ...
        """

        request = "/{}/{}/raw/last/{}".format(route, code, last)
        headers = self.get_auth_headers(request)
        url = "{}{}".format(self.baseurl, request)
        # print url
        try:
            r = requests.get(url, headers=headers, verify=False)
        except:
            print('ERRORE COMUNICAZIONE')
            return False
        else:
            if r.status_code == 200:
                return r.json()
            return False

    def get_df(self, code, last=1):
        d = self._get_data(code, route='pretty', last=last)
        if d is False:
            return d
        self.df = self._todf(d)

        # merge level names: e.g. variable|aggregate
        self.df.columns = self.df.columns.map('|'.join).str.strip('|')

        # localize datetime index
        self.df.index = self.df.index.tz_localize(self.tz)
        return self.df




from django.core.management.base import BaseCommand

# from meteo.pgutils import load_data
from measurements.settings import SOURCE_AUTH
from measurements.sources.pessl import PesslAPI
from measurements.models import SourceType
from measurements.utils import get_serie, load_serie

PARAMETER_MAP = {"Precipitation|sum": "Precipitation", #
                 "Leaf Wetness|time": "LeafWet", #
                 'HC Air temperature|avg': "AtTemp", # http://vocab.nerc.ac.uk/collection/P01/current/CTMPZZ01/
                 "HC Relative humidity|avg": "RelHumidity", #http://vocab.nerc.ac.uk/collection/P01/current/CHUMZZ01/
                 "Dew Point|avg": "DewPoint", # http://vocab.nerc.ac.uk/collection/P01/current/CDEWZZ01/
                 "Wind speed|avg": "WindSpeed", # http://vocab.nerc.ac.uk/collection/P01/current/EWSBZZ01/
                 }


class Command(BaseCommand):
    help = "Command to import PESSL data"

    def handle(self, *args, **options):
        ps = SourceType.objects.get(code='pessl')
        for s in ps.station_set.filter(status='active'):
            keys = SOURCE_AUTH['pessl'][s.network.code]
            print(keys)
            pesslapi = PesslAPI(keys['public_key'],
                                keys['private_key'])
            df = pesslapi.get_df(s.code, 100)

            for k, v in PARAMETER_MAP.items():
                if k in df.columns:
                    serie = get_serie(s, v)
                    load_serie(df[k].copy(), serie.id)



import os
import datetime
import hmac
import hashlib
import urllib3
urllib3.disable_warnings()
import requests
import pandas as pd

import xml.etree.ElementTree as et

from measurements.sources.base import BaseSource
from measurements.utils import strong_float

class MttAPI(BaseSource):
    def __init__(self, tz='Etc/GMT-1'):
        self.baseurl = "http://dati.meteotrentino.it/service.asmx/ultimiDatiStazione?"
        self.tz = tz

    def get_df(self, code, last=1):
        url = self.baseurl + 'codice=' + code
        r = requests.get(url)
        print(url)

        self.df = self.parse_xml(r.text)

        self.df.index = pd.to_datetime(self.df.index)

        # localize datetime index
        self.df.index = self.df.index.tz_localize(self.tz)
        return self.df

    def parse_xml(self, xml):
        root = et.fromstring(xml)

        def get_data(root, parent, childs):
            data = []
            namespaces = {'mtt': 'http://www.meteotrentino.it/'}
            parent_node = root.find(parent, namespaces)
            if not parent_node:
                return None
            for r in parent_node.findall(childs, namespaces):
                row = {}
                for c in r.getchildren():
                    k = c.tag[30:]
                    v = c.text
                    if k == 'data':
                        row[k] = v
                    else:
                        row[k] = strong_float(v)
                data.append(row)
            if len(data) == 0:
                return None
            df = pd.DataFrame(data)
            df.set_index('data', inplace=True)
            return df

        data_names = (
            ('temperature', 'temperatura_aria'),
            ('precipitazioni', 'precipitazione'),
            ('venti', 'vento_al_suolo'),
            ('radiazione', 'radiazioneglobale'),
            ('umidita_relativa', 'umidita_relativa'),
        )

        dfs = []
        for name in data_names:
            df = get_data(root,
                          "mtt:{}".format(name[0]),
                          "mtt:{}".format(name[1]),)
            if df is not None:
                dfs.append(df)
        dftot = pd.concat(dfs, axis=1)

        # convert km/m to m/s (if exists)
        if 'v' in dftot.columns:
            dftot['v'] = dftot['v'] / 3.6

        return dftot



import json
from django.shortcuts import render
from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt
from .models import Sensor, Station, Parameter, Serie, Measure
from basicauth.decorators import basic_auth_required
from django.http import HttpResponse
from datetime import datetime

from .utils import get_serie

#
# @basic_auth_required(
#     target_test=lambda request: not request.user.is_authenticated
# )

TS_NS = 1000000000 # to convert from nanoseconds to seconds
SERIE_FIELDS = set(['station', 'sensor', 'parameter'])

@csrf_exempt
def write(request):
    # rjson = json.loads(request.body)
    # _write(rjson)
    # return JsonResponse(rjson, safe=False)
    # return HttpResponse(request.body)
    res = write_data(request.body)
    # split res


    return JsonResponse(res)

def write_data(body):
    data = []
    for l in body.decode().strip().split("\n"):
        data.append(parse_line(l))
    stats = _write(data)
    return stats

def parse_line(line):
    # TODO: using regexp
    measure, others = line.split(",", 1)
    _o = others.split(" ")
    if len(_o) == 3:
        tags, fields, _timestamp = _o
        _timestamp = datetime.utcfromtimestamp(int(_timestamp)/TS_NS)
    elif len(_o) == 2:
        tags, fields = _o
        _timestamp = datetime.utcnow()
    r = {'m': measure,
            'ts': _timestamp,
            't': dict(x.split('=') for x in tags.split(',')),
            'f': dict(x.split('=') for x in fields.split(','))
            }

    # check for valid serie
    keys = set(r['t'].keys())
    if not 'serie' in keys and not SERIE_FIELDS <= keys:
        raise ValueError("Incomplete serie definition")

    return r


def _write(rjson):
    stats = {'updated': 0,
             'created': 0}
    for r in rjson:
        tags = r['t']
        fields = r['f']


        _station = tags['station']
        _parameter = tags['parameter']
        _sensor = tags.get('sensor', 'unknown')

        serie = get_serie(_station, _parameter, _sensor)

        timestamp = r.get('ts')
        print("TS", timestamp)
        value = fields.get('value')
        measure, created = Measure.objects.get_or_create(serie=serie,
                                                         timestamp=timestamp,
                                                         value=value)
        if created:
            stats['created'] += 1
        else:
            stats['updated'] += 1
    return stats


from setuptools import setup, find_packages

setup(
    name='django-measurements',
    version='0.11',
    # packages=['measurements'],
    packages=find_packages(),
    url='',
    license='GPL3',
    author='Stefano Menegon',
    author_email='stefano.menegon@cnr.it',
    description='', install_requires=['django', 'numpy', 'pandas',
                                      'urllib3', 'requests', 'pycryptodome',
                                      'django-postgres-extra', 'django_pandas',
                                      'geojson', 'pysal', 'colorbrewer',
                                      'django-basicauth']
)



from django.contrib import admin

from .models import Parameter, Sensor, Location, Serie, Measure, Network, SourceType, Station


class ParameterAdmin(admin.ModelAdmin):
    pass


class SensorAdmin(admin.ModelAdmin):
    pass


class LocationAdmin(admin.ModelAdmin):
    pass


class NetworkAdmin(admin.ModelAdmin):
    pass


class SerieAdmin(admin.ModelAdmin):
    list_display = ('station', 'sensor', 'parameter', 'height')
    list_filter = ('station', 'sensor', 'parameter')


class StationAdmin(admin.ModelAdmin):
    pass


class SourceTypeAdmin(admin.ModelAdmin):
    pass


class MeasureAdmin(admin.ModelAdmin):
    list_display = ('serie', 'timestamp', 'value')


admin.site.register(Parameter, ParameterAdmin)
admin.site.register(Sensor, SensorAdmin)
admin.site.register(Location, LocationAdmin)
admin.site.register(Network, NetworkAdmin)
admin.site.register(Serie, SerieAdmin)
admin.site.register(Station, StationAdmin)
admin.site.register(SourceType, SourceTypeAdmin)
admin.site.register(Measure, MeasureAdmin)
